use crate::fs::fs_factory::FilesystemDesc;
use crate::types::Result;
use crate::errors::Error;

use crate::unix::mounts::{MountHandler, MountDesc, FsType, MountOpt};
use crate::unix::loop_handler::LoopHandler;

use std::path::PathBuf;

/*
 * Descriptor for a Tmpfs ephemeral filesystem
 */
struct TmpfsDesc {
    mount_desc: Option<MountDesc>
}

impl FilesystemDesc for TmpfsDesc {
    fn from_path(_path: PathBuf) -> Result<Self> { Ok(TmpfsDesc{mount_desc: None}) }
    fn from_mem(_data: Vec<u8>) -> Result<Self> { Ok(TmpfsDesc{mount_desc: None}) }
    fn format(&self, _handler: &MountHandler, _size: usize) -> Result<()> { Ok(()) }

    fn get_path(&self) -> PathBuf { PathBuf::new() }
    fn mount(&mut self, handler: &MountHandler, _loopctl: &LoopHandler, path: PathBuf)
             -> Result<()> {
        let temp_mount = MountDesc {
            source: "tmpfs".to_string(),
            target: path.to_str().unwrap().to_string(),
            partition_type: FsType::Tmpfs,
            options: vec![MountOpt::Noatime],
            data: None,
        };

        handler.mount(temp_mount.clone())?;
        self.mount_desc = Some(temp_mount);
        
        Ok(())
    }

    fn umount(&mut self, handler: &MountHandler, _loopctl: &LoopHandler) -> Result<()> {
        if self.mount_desc.is_none() {
            return Err(Error::FilesystemDescError("not mounted".into()));
        }
        handler.umount(self.mount_desc.clone().unwrap(), true, false)?;
        self.mount_desc = None;
        Ok(())
    }
}
