use crate::namespace::{Namespace, NamespaceOption};
use crate::types::Pid;

use serde::ser::SerializeMap;
use serde::{Serialize, Serializer};

pub type NamespaceEntry = (Pid, NamespaceOption, Namespace);
pub type NamespaceBundle = Vec<NamespaceEntry>;

pub struct NamespaceBundleStruct(pub NamespaceBundle);

mod namespace_bundle_types {
    use crate::namespace::{Namespace, NamespaceOption};
    use crate::types::Pid;

    use serde::ser::SerializeMap;
    use serde::{Serialize, Serializer};

    pub use std::cell::RefCell;
    pub use std::collections::HashMap;
    pub use std::rc::Rc;

    pub type MutNSMapPidRepr = Rc<RefCell<Box<Vec<Pid>>>>;
    pub type MutNSMapOptEntryRepr = Rc<RefCell<HashMap<Namespace, MutNSMapPidRepr>>>;
    pub type MutNSMapRepr = Rc<RefCell<HashMap<NamespaceOption, MutNSMapOptEntryRepr>>>;

    pub struct NSMapOptEntryRepr(pub MutNSMapOptEntryRepr);

    impl Serialize for NSMapOptEntryRepr {
        fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
        where
            S: Serializer,
        {
            let mut map = serializer.serialize_map(Some(self.0.borrow().len()))?;
            for (ns, pidv) in self.0.borrow().iter() {
                map.serialize_entry(ns.get_id(), &*pidv.borrow())?;
            }
            map.end()
        }
    }
}

impl Serialize for NamespaceBundleStruct {
    fn serialize<S>(&self, serializer: S) -> std::result::Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        use namespace_bundle_types::*;

        let namespace_mut_map: MutNSMapRepr = Rc::new(RefCell::new(HashMap::new()));

        for entry in (&self.0).into_iter() {
            let pid = entry.0.clone();
            let opt = entry.1;
            let ns = entry.2;

            let option;

            if namespace_mut_map.try_borrow().unwrap().contains_key(&opt) {
                option = namespace_mut_map
                    .try_borrow()
                    .unwrap()
                    .get(&opt)
                    .unwrap()
                    .to_owned();
            } else {
                option = Rc::new(RefCell::new(HashMap::new()));
                namespace_mut_map
                    .try_borrow_mut()
                    .unwrap()
                    .insert(opt, option.clone());
            };

            let pid_list;

            if option.try_borrow().unwrap().contains_key(&ns) {
                pid_list = option.try_borrow().unwrap().get(&ns).unwrap().to_owned();
            } else {
                pid_list = Rc::new(RefCell::new(Box::new(vec![])));
                option
                    .try_borrow_mut()
                    .unwrap()
                    .insert(ns, pid_list.clone());
            }

            if !pid_list.try_borrow().unwrap().contains(&pid) {
                pid_list.try_borrow_mut().unwrap().insert(0, pid);
            }
        }

        let mut map = serializer.serialize_map(Some(self.0.len()))?;
        for (o, er) in namespace_mut_map.try_borrow().unwrap().iter() {
            let oer = NSMapOptEntryRepr(er.to_owned());
            map.serialize_entry(&o.to_string(), &oer)?;
        }
        map.end()
    }
}
