use crate::state::{AbstractState, FsStore};
use crate::types::Result;

use std::path::PathBuf;
use std::str::FromStr;

const GLOBAL_STATE_TYPE: &str = "global_state";
const FS_STORE_PATH: &str = "root-filesystems";

pub struct GlobalState {
    base_path: PathBuf,

    pub fs_store: FsStore,
    // network_state: u64,

    // box_state: u64
}

impl AbstractState for GlobalState {
    fn desc() -> String { GLOBAL_STATE_TYPE.to_string() }

    fn empty_variant() -> GlobalState {
        GlobalState {
            base_path: PathBuf::new(),
            fs_store: FsStore::empty_variant()
        }
    }

    fn base_path(&self) -> PathBuf {
        self.base_path.clone()
    }
    fn state_path(&self) -> PathBuf {
        PathBuf::from_str("/").unwrap()
    }

    fn open_internal(path: PathBuf, init: Option<GlobalState>) -> Result<GlobalState> {
        let fs_store_path = path.join(FS_STORE_PATH.to_string());

        Ok(GlobalState {
            base_path: path,
            fs_store: FsStore::open(fs_store_path, init.map(|s| s.fs_store))?
        })
    }
}
