use crate::errors;

use serde::Serialize;

use std::error::Error;
use std::result::Result as BaseResult;

use std::fmt::Debug;

pub type ResultOpen<T> = BaseResult<T, Box<dyn Error>>;
pub type Result<T> = BaseResult<T, errors::Error>;

pub type NsId = u32;

#[derive(Debug, Clone, Hash, PartialEq, Eq, Serialize)]
pub struct Pid {
    pub pid: u64,
    pub desc: String,
}

impl ToString for Pid {
    fn to_string(&self) -> String {
        self.pid.to_string()
    }
}

pub fn result_flatten<T: Copy, E>(
    c: std::result::Result<Vec<T>, E>,
    tr: std::result::Result<T, E>,
) -> std::result::Result<Vec<T>, E> {
    match (c, tr) {
        (Ok(_), Err(e)) => Err(e),
        (Ok(tv), Ok(t)) => Ok({
            let a = [tv, vec![t]].concat();
            a
        }),
        (Err(e), _) => Err(e),
    }
}

pub trait Expectable<T, E> {
    fn xpect(self, msg: &str) -> BaseResult<T, E>;
    fn spect(self, msg: String) -> BaseResult<T, E>;
}

impl<T, E: Debug> Expectable<T, E> for BaseResult<T, E> {
    fn xpect(self, msg: &str) -> BaseResult<T, E> {
        self.map_err(|e| {
            log::error!("{} : {:?}", msg, e);
            e
        })
    }

    fn spect(self, msg: String) -> BaseResult<T, E> {
        self.map_err(|e| {
            log::error!("{} : {:?}", msg, e);
            e
        })
    }
}
